#include <stdio.h>
#include <string.h>

#include <errno.h>

#include <sys/ioctl.h> // get network card id
#include <net/if.h>
#include <sys/socket.h>
#include <arpa/inet.h> /* htons */

#include <linux/if_packet.h>
#include <net/ethernet.h>


int main(int argc, char **argv) {
    if(argc != 2) {
        fprintf(stderr, "Invalid arguments !, you need to specify a network card and a target mac address.\n");
        printf("%s <inet> <mac>\n", argv[0]);
        return 1;
    }

    // We check if the passed name is valid size
    if(strlen(argv[1]) > IFNAMSIZ) {
        fprintf(stderr, "Invalid name !");
        return 1;
    }

    struct ifreq ifr;
    strcpy(ifr.ifr_name, argv[1]);

    int s;
    if((s = socket(AF_PACKET, SOCK_DGRAM, htons(0x0842))) == -1) {
        fprintf(stderr, "An error has occured with socket.\n");
        return 1;
    }

    // We catch the network card info (ioctl)
    if(ioctl(s, SIOCGIFADDR, &ifr) == -1) {
        fprintf(stderr, "An error has occured with ioctl. Is it a valid network interface name?\n");
        return 1;
    }

    struct sockaddr_ll linkl;
    memset(&linkl, 0, sizeof(struct sockaddr_ll));

    linkl.sll_family = AF_PACKET;
    linkl.sll_ifindex = ifr.ifr_ifindex;
    linkl.sll_protocol = htons(0x0842);
    linkl.sll_halen = ETHER_ADDR_LEN;
    memset(&linkl.sll_addr, 0xff, ETHER_ADDR_LEN);


    unsigned char data[sizeof(char) * 6 * 17];
    memset(data, 0xff, sizeof(char) * 6);

    // experimental     E0D55E8410D0
    const char ethaddr[] = { 0xe0, 0xd5, 0x5e, 0x84, 0x10, 0xd0 };
    for(int i = 1; i <= 16; i++) {
        memcpy(data + i * 6, ethaddr, sizeof(ethaddr));
    }

    if(sendto(s, (void*)data, sizeof(data), 0, (struct sockaddr*)&linkl, sizeof(struct sockaddr_ll)) == -1) {
        fprintf(stderr, "sendto failed: %s\n", strerror(errno));
    }
}